import React from 'react'
import ReactDOM from 'react-dom'
import SimpleMDE from 'simplemde'
import PropTypes from 'prop-types'
import ques from '../lib/ques'
import { getButtons } from './buttons'

const defaultOptions = {
  autoDownloadFontAwesome: false,
  forceSync: true,
  parsingConfig: {
    allowAtxHeaderWithoutSpace: true,
  },
  previewRender: function(content) {
    return ques.render(content)
  },
  spellChecker: false,
  toolbar: [
    'bold', 'italic', 'heading',
    '|', 'code', 'latex', 'quote', 'unordered-list', 'ordered-list',
    '|', 'link', 'upload-image', 'upload-file', 'table',
    '|', 'preview', 'side-by-side', 'fullscreen',
  ]
}

export default class Editor extends React.Component {
  set value(data) {
    if (this.text() !== data) {
      this.text(data)
    }
  }
  get value() {
    return this.text()
  }
  componentDidMount() {
    let toolbar = this.props.toolbar || defaultOptions.toolbar
    if (this.props.guide) {
      toolbar = toolbar.concat(['|', 'guide'])
    }
    const options = Object.assign({}, defaultOptions, {
      element: ReactDOM.findDOMNode(this.refs.inputArea),
      toolbar: getButtons(toolbar, {
        guide: this.props.guide,
        lng: this.props.lng,
        uploader: this.props.uploader,
      }),
    }, this.props.simplemde)
    this.simplemde = new SimpleMDE(options)
    if (this.props.value) {
      this.text(this.props.value)
    }
    this.simplemde.codemirror.on('change', () => {
      if (this.props.onChange) this.props.onChange.call(this, this.text())
    })
  }
  shouldComponentUpdate(nextProps) {
    this.update(nextProps)
    return false
  }
  on(...args) {
    return this.simplemde.codemirror.on.apply(this.simplemde.codemirror, args)
  }
  off(...args) {
    return this.simplemde.codemirror.off.apply(this.simplemde.codemirror, args)
  }
  text(value) {
    if (this.simplemde.isPreviewActive() && value !== undefined) {
      const wrapper = this.simplemde.codemirror.getWrapperElement()
      const preview = wrapper.lastChild
      preview.innerHTML = ques.render(value)
    }
    return this.simplemde.value.call(this.simplemde, value)
  }
  html() {
    return ques.render(this.text())
  }
  updateValue(nextProps) {
    if (nextProps.value !== this.text()) {
      this.text(nextProps.value)
    }
  }
  updateToolbar(nextProps) {
    if (JSON.stringify(this.props.toolbar) !== JSON.stringify(nextProps.toolbar)
      || this.props.guide !== nextProps.guide
      || this.props.lng !== nextProps.lng
      || this.props.uploader !== nextProps.uploader
    ) {
      let toolbar = nextProps.toolbar || defaultOptions.toolbar
      const simplemde = this.simplemde
      const wrapper = simplemde.codemirror.getWrapperElement()

      if (nextProps.guide) {
        toolbar = toolbar.concat(['|', 'guide'])
      }
      const buttons = getButtons(toolbar, {
        lng: nextProps.lng,
        uploader: nextProps.uploader,
        guide: nextProps.guide,
      })

      if (simplemde.gui.toolbar) {
        wrapper.parentNode.removeChild(simplemde.gui.toolbar)
      }
      simplemde.gui.toolbar = simplemde.createToolbar(buttons)
    }
  }
  updateName(nextProps) {
    const inputArea = ReactDOM.findDOMNode(this.refs.inputArea)
    if (inputArea.name !== nextProps.name) {
      inputArea.name = nextProps.name
    }
  }
  update(nextProps) {
    this.updateToolbar(nextProps)
    this.updateValue(nextProps)
    this.updateName(nextProps)
  }
  render() {
    return (
      <div className="pta-editor">
        <textarea ref="inputArea" name={this.props.name}/>
      </div>
    )
  }
}

Editor.propTypes = {
  guide: PropTypes.string,
  lng: PropTypes.string,
  name: PropTypes.string,
  onChange: PropTypes.func,
  simplemde: PropTypes.object,
  toolbar: PropTypes.array,
  uploader: PropTypes.func,
  value: PropTypes.string,
}

Editor.defaultProps = {
  simplemde: {},
}

export const render = Editor.render = ques.render
